/*
 * Parameter file
 *
 * Robert Johansson <robert@riken.jp>
 *
 * $Id: param.h,v 1.4 2008-10-23 07:15:48 rob Exp $
 */

#include <math.h>

#include <gsl/gsl_matrix.h>
#include "quantum_system.h"

#ifndef PARAM_H
#define PARAM_H

typedef void*(*hamiltonian_func_t)(void *, void *);
typedef int(*hamiltonian_td_func_t)(void *, void *, void *, double, void *);
typedef double(*ho_w_func_t)(double, void*);


typedef struct {
	quantum_system *qs;
	
	/* -- time evolution -- */
	double Ti;
	double Tf;
	double dT;

	double x;

	char simsig[1024];
	
	/* --- Temperature in frequency unit --- */
	double wT;

	/* -- Hamiltonian -- */
	int N;
	int n_offset;

	double *epsilon;
	double *delta;
	double **lambda;

	gsl_matrix_complex *H0;
	gsl_matrix_complex *H1;
	gsl_matrix_complex *H_t;


	/* -- dissipation -- */
	gsl_matrix_complex *do_a[10];
	gsl_matrix_complex *do_ad[10];
	gsl_matrix_complex *do_aad[10];
	gsl_matrix_complex *do_ada[10];

	gsl_matrix_complex *ws1;
	gsl_matrix_complex *ws2;
	
	double do_g1[10];
	double do_g2[10];

	int do_n;

	/* --- continuous basis transform --- */
	gsl_matrix_complex *dSSt;
	ho_w_func_t ho_w_func;
	int cont_basis_transform;
	
	/* --- Harmonic oscillator --- */
	double *ho_w;
	double ho_g;
	double ho_g2;
	double g_qc;
	double g_cr;

	gsl_matrix_complex *N_op[10];
	gsl_matrix_complex *a, *ad, *b, *bd, *b2, *bd2, *rho_f, *A, *B;
	gsl_matrix_complex *n_op, *n_op_2, *x1, *x1_2, *x2, *x2_2;

	/* -- driving field -- */
	
	double f_A;
	double f_w;
	double phi;

	double Nexpt[2];
	
	/* --- hamiltonians --- */
	hamiltonian_func_t H0_func;
	hamiltonian_func_t H1_func;
	hamiltonian_td_func_t Ht_func;


	/* --- Dicke model stuff --- */
	double j;
	double *J_w;
	double g_ac, g_sc;
	
} solver_param;

#endif
