/*
 * Functions related to definition of quantum systems (composed of subsystems
 * with different sizes) and enummeration of states
 *
 * Robert Johansson <robert@riken.jp>
 *
 * $Id: quantum_system.h,v 1.1.1.1 2008-09-03 09:36:41 rob Exp $
 */

#include <math.h>

#ifndef QUANTUM_SYSTEM_H
#define QUANTUM_SYSTEM_H

#define NQS_MAX 20

typedef int quantum_system_state_vector_t[NQS_MAX];

typedef struct {
	/* system 0: 2 levels
	 * system 1: 2 levels
	 * system 3: 10 levels
	 * etc.
	 */

	/* Number of subsystems in the quantum system */
	int nsubsys;

	/* Number of states of each subsystem */
	int *nstates;
} quantum_system;


quantum_system * quantum_system_new();
quantum_system * quantum_system_copy(quantum_system *qs);
void quantum_system_print(quantum_system *qs);
void	quantum_system_free(quantum_system *qs);
void    quantum_system_add(quantum_system *qs, int M);
int 	quantum_system_nstates(quantum_system *qs);
void 	quantum_system_state_vector(quantum_system *qs, int qsn, quantum_system_state_vector_t qsv);
int 	quantum_system_state_number(quantum_system *qs, quantum_system_state_vector_t qsv);


#endif
